"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Locations = void 0;
var armHelper_1 = require("../armHelper");
var constants_1 = require("../../Helpers/constants");
var urlParams_1 = require("./urlParams");
var Locations;
(function (Locations) {
    "use strict";
    /**
     * Initializes the routes.
     *
     * @param app The express app.
     */
    function init(app) {
        app.get("/locations", function (req, res, next) {
            armHelper_1.ArmHelper.handleRequest(req, res, next, function (armManager) {
                armHelper_1.ArmHelper.proxyReq(req, res, []).then(function (actualLocations) {
                    actualLocations = actualLocations || [];
                    var mockLocations = ((armManager && armManager.getLocations()) || []).filter(function (mockLocation) {
                        return !actualLocations.some(function (actualLocation) { return actualLocation.name.toLowerCase() === mockLocation.name.toLowerCase(); });
                    });
                    var response = {
                        value: actualLocations.concat(mockLocations)
                    };
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(response);
                }).catch(function (err) {
                    console.error(err);
                });
            });
        });
        app.get("/subscriptions/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/locations"), function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            if (mockSubscriptionId) {
                var armManager = req.app.armManager;
                var response = {
                    value: armManager.getLocations()
                };
                res.status(constants_1.Constants.HttpStatusCode.Ok).json(response);
            }
            else {
                next();
            }
        });
    }
    Locations.init = init;
})(Locations = exports.Locations || (exports.Locations = {}));
//# sourceMappingURL=locations.js.map