"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Providers = void 0;
var armHelper_1 = require("../armHelper");
var constants_1 = require("../../Helpers/constants");
var urlParams_1 = require("./urlParams");
var utilities_1 = require("../../Helpers/utilities");
var Providers;
(function (Providers) {
    "use strict";
    /**
     * Initializes the routes.
     *
     * @param app The express app.
     */
    function init(app) {
        app.get("/providers", function (req, res, next) {
            armHelper_1.ArmHelper.handleRequest(req, res, next, function (armManager) {
                armHelper_1.ArmHelper.proxyReq(req, res, []).then(function (providers) {
                    var mockProviders = !armManager ? [] : armManager.getProviders();
                    var response = {
                        value: providers.concat(mockProviders)
                    };
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(response);
                }).catch(function (err) {
                    console.error(err);
                });
            });
        });
        app.get("/providers/:providerId", function (req, res, next) {
            armHelper_1.ArmHelper.handleRequest(req, res, next, function (armManager) {
                var providerId = req.params["providerId"].toLowerCase();
                var provider = utilities_1.Utilities.findInArr(armManager.getProviders(), function (p) { return p.namespace.toLowerCase() === providerId; });
                if (provider) {
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(provider);
                }
                else {
                    next();
                }
            });
        });
        app.get("/subscriptions/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/providers"), function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            if (mockSubscriptionId) {
                var armManager = req.app.armManager;
                res.status(constants_1.Constants.HttpStatusCode.Ok).json(armManager.getProviders());
            }
            else {
                next();
            }
        });
        app.get("/subscriptions/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/providers/:providerId"), function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            if (mockSubscriptionId) {
                var providerId = req.params["providerId"].toLowerCase();
                var armManager = req.app.armManager;
                var provider = utilities_1.Utilities.findInArr(armManager.getProviders(), function (p) { return p.namespace.toLowerCase() === providerId; });
                if (provider) {
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(provider);
                }
                else {
                    res.status(constants_1.Constants.HttpStatusCode.NotFound).send("Provider not found");
                }
            }
            else {
                next();
            }
        });
        app.get("/subscriptions/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/providers/microsoft.insights/eventtypes/management/digestEvents"), function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            if (mockSubscriptionId) {
                res.status(constants_1.Constants.HttpStatusCode.Ok).json({ value: [] });
            }
            else {
                next();
            }
        });
    }
    Providers.init = init;
})(Providers = exports.Providers || (exports.Providers = {}));
//# sourceMappingURL=providers.js.map