"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResourceGroups = void 0;
var constants_1 = require("../../Helpers/constants");
var urlParams_1 = require("./urlParams");
var utilities_1 = require("../../Helpers/utilities");
var ResourceGroups;
(function (ResourceGroups) {
    "use strict";
    /**
     * Initializes the routes.
     *
     * @param app The express app.
     */
    function init(app) {
        app.get("/subscriptions/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/resourcegroups"), function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            if (mockSubscriptionId) {
                var armManager = req.app.armManager;
                var resourceGroups = armManager.getResourceGroups(mockSubscriptionId);
                res.status(constants_1.Constants.HttpStatusCode.Ok).json({
                    value: resourceGroups
                });
            }
            else {
                next();
            }
        });
        app.route("/subscriptions/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/resourcegroups/:").concat(urlParams_1.UrlParams.Params.resourceGroupId))
            .get(function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockResourceGroupId = req[urlParams_1.UrlParams.Params.resourceGroupId];
            if (mockSubscriptionId) {
                var armManager = req.app.armManager;
                var resourceGroup = armManager.getResourceGroup(mockSubscriptionId, mockResourceGroupId);
                res.status(constants_1.Constants.HttpStatusCode.Ok).json(resourceGroup);
            }
            else {
                next();
            }
        })
            .patch(function (req, res, next) {
            // TODO
            res.sendStatus(constants_1.Constants.HttpStatusCode.NotImplemented);
        })
            .delete(function (req, res, next) {
            // TODO
            res.sendStatus(constants_1.Constants.HttpStatusCode.NotImplemented);
        });
        app.route("/subscriptions/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/resourcegroups/:").concat(urlParams_1.UrlParams.Params.resourceGroupId, "/deployments"))
            .get(function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            if (mockSubscriptionId) {
                res.status(constants_1.Constants.HttpStatusCode.Ok).json({
                    value: []
                });
            }
            else {
                next();
            }
        });
        app.route("/subscriptions/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/resourcegroups/:").concat(urlParams_1.UrlParams.Params.resourceGroupId, "/providers/microsoft.authorization/permissions"))
            .get(function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            if (mockSubscriptionId) {
                res.status(constants_1.Constants.HttpStatusCode.Ok).json({
                    value: [
                        {
                            actions: ["*"],
                            notActions: []
                        }
                    ],
                    nextLink: null
                });
            }
            else {
                next();
            }
        });
    }
    ResourceGroups.init = init;
    function logResourceEvent(subscriptionId, resourceGroupId, resourceId, resourceType, action, description, status) {
        var resourceProvider = resourceType.split('/')[0].toLowerCase();
        var digestEvent = {
            id: utilities_1.Utilities.newGuid(),
            description: description,
            eventChannels: "Operation",
            operationName: {
                value: resourceType + "/" + action
            },
            subscriptionId: subscriptionId,
            resourceGroupName: resourceGroupId,
            resourceProviderName: {
                value: resourceProvider
            },
            resourceUri: resourceId,
            status: {
                value: status || "succeeded"
            },
            eventTimestamp: (new Date()).toString()
        };
    }
})(ResourceGroups = exports.ResourceGroups || (exports.ResourceGroups = {}));
//# sourceMappingURL=resourceGroups.js.map