import * as ArmModel from "../Models/armModel";
import * as html5 from "../../Helpers/html5";
import StringMap = html5.StringMap;
export interface MappedArmData<T> {
    mappedData?: T;
}
export interface Lock extends ArmModel.ArmLockData {
    /**
     * The name given to the lock
     */
    name: string;
}
export interface Subscription extends MappedArmData<ArmModel.Subscription> {
    /**
     * Friendly name of the azure subscription.
     */
    displayName: string;
    /**
     * Id of the azure subscription.
     */
    subscriptionId: string;
    /**
     * The subscription state.
     */
    state: string;
    /**
     * The subscription policies. May be null.
     */
    subscriptionPolicies?: SubscriptionPolicies;
    /**
     * The list of resource groups.
     */
    resourceGroups?: ResourceGroup[];
    /**
     * The list of lcoks.
     */
    locks?: Lock[];
}
/**
 * Data contract for Azure subscription policies.
 */
export interface SubscriptionPolicies {
    /**
     * The subscription location placement id.
     */
    locationPlacementId: string;
    /**
     * The subscription quota id.
     */
    quotaId: string;
}
export interface ResourceGroup extends MappedArmData<ArmModel.ResourceGroup> {
    /**
     * Resource group name.
     */
    name: string;
    /**
     * Resource group tags.
     */
    tags: StringMap<string>;
    /**
     * Resource group properties.
     */
    properties: ResourceGroupProperties;
    /**
     * Resource group location.
     */
    location: string;
    /**
     * The list of resources in the resource group.
     */
    resources?: Resource[];
    /**
     * The list of lcoks.
     */
    locks?: Lock[];
}
/**
 * Resource group properties.
 */
export interface ResourceGroupProperties {
    /**
     * Resource group lock state.
     */
    lockState: string;
    /**
     * Resource group provisioning state.
     */
    provisioningState: string;
}
export interface Resource extends MappedArmData<ArmModel.Resource> {
    /**
     * Resource name.
     */
    name: string;
    /**
     * Resource location.
     */
    location: string;
    /**
     * Resource tags.
     */
    tags: StringMap<string>;
    /**
     * Resource type.
     */
    type: string;
    /**
     * Resource kind.
     */
    kind: string;
    /**
     * List of nested resources
     */
    nestedResources?: NestedResource[];
    /**
     * The list of lcoks.
     */
    locks?: Lock[];
}
export interface NestedResource extends MappedArmData<ArmModel.Resource> {
    /**
     * Resource name.
     */
    name: string;
    /**
     * Resource tags.
     */
    tags: StringMap<string>;
    /**
     * Mested resource type.
     */
    type: string;
}
