import * as Q from "q";
import * as ArmModel from "./Models/armModel";
import * as MockModel from "./MockModels/mockModel";
export declare module ArmManager {
    /**
     * Defines structore of mock data.
     */
    interface MockData {
        subscriptions?: MockModel.Subscription[];
        providers?: ArmModel.Provider[];
    }
    /**
     * The mock azure resource manager
     */
    class Manager {
        private data;
        private events;
        /**
         * Initializes the mock data. Call this with empty data to reset state.
         *
         * @param data The mock data.
         * @return A promise that is resolved when the data is initialized.
         */
        initializeMockData(data: MockData): Q.Promise<any>;
        /**
         * Gets a subscription based on id from the mock data.
         *
         * @param subscriptionId The subscription GUID/ID.
         */
        getEvents(subscriptionId: string): ArmModel.EventData[];
        /**
         * Gets the list of providers.
         */
        getProviders(): ArmModel.Provider[];
        /**
         * Gets list of mock locations.
         */
        getLocations(): ArmModel.Location[];
        /**
         * Gets the list of mock subscriptions.
         */
        getSubscriptions(): ArmModel.Subscription[];
        /**
         * Gets a subscription based on id from the mock data.
         *
         * @param subscriptionId The subscription GUID/ID.
         */
        getSubscription(subscriptionId: string): ArmModel.Subscription;
        /**
         * Gets the list of mock resource groups under a subscription.
         *
         * @param subscriptionId The subscription GUID/ID.
         */
        getResourceGroups(subscriptionId: string): ArmModel.ResourceGroup[];
        /**
         * Gets the resource group based on id from the mock data.
         *
         * @param subscriptionId The subscription GUID/ID.
         * @param resourceGroupId The resource group name.
         */
        getResourceGroup(subscriptionId: string, resourceGroupId: string): ArmModel.ResourceGroup;
        /**
         * Gets all resources under a subscription and resource group
         *
         * @param subscriptionId The subscription GUID/ID.
         * @param resourceGroupId The resource group name.
         */
        getResources(subscriptionId: string, resourceGroupId: string): ArmModel.Resource[];
        /**
         * Gets a resource under a subscription and resource group.
         *
         * @param subscriptionId The subscription GUID/ID.
         * @param resourceGroupId The resource group name.
         * @param type The resource type.
         * @param resourceName The resource name.
         */
        getResource(subscriptionId: string, resourceGroupId: string, type: string, resourceName: string): ArmModel.Resource;
        /**
         * Publishes a subscription event.
         *
         * @param subscriptionId The subscription GUID/ID.
         * @param data The event data.
         */
        publishEvent(subscriptionId: string, data: ArmModel.EventData): void;
        getSubscriptionData(subscriptionId: string): MockModel.Subscription;
        getResourceGroupData(subscriptionId: string, resourceGroupId: string): MockModel.ResourceGroup;
        getResourceData(subscriptionId: string, resourceGroupId: string, type: string, resourceName: string): MockModel.Resource;
        getNestedResourceData(subscriptionId: string, resourceGroupId: string, type: string, resourceName: string, nestedType: string, nestedResourceName: string): MockModel.NestedResource;
    }
}
