import * as express from "express";
import * as Q from "q";
import { ArmManager } from "./armManager";
import { HttpsProxy } from "../HttpsProxy/httpsProxy";
export declare module ArmProxy {
    interface ArmExpressServer extends HttpsProxy.ExpressServer {
        /**
         * The instance of arm manager.
         */
        armManager?: ArmManager.Manager;
    }
    /**
     * Creates an Arm proxy instance and setups firewal and certificates.
     *
     * @param armEndpoint The arm endpoint to proxy.
     * @param serverPort The local server port.
     * @param armManager The arm mock manager.
     * @param enablePassthrough If to enable passthrough to actual arm.
     * @return A promise that completes when local proxy is setup.
     */
    function create(armEndpoint: string, serverPort: number, armManager?: ArmManager.Manager, customRoutes?: (app: express.Express) => void, enablePassthrough?: boolean): Q.Promise<HttpsProxy.ProxyServer>;
    /**
     * Dispose local proxy
     *
     * @param armEndpoint The arm endpoint to proxy.
     * @param serverPort The local server port.
     * @param enablePassthrough If to enable passthrough to actual arm.
     * @return A promise that completes when local proxy is setup.
     */
    function dispose(proxy: HttpsProxy.ProxyServer): Q.Promise<any>;
}
