"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Utilities = void 0;
var Utilities;
(function (Utilities) {
    var hexValues = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F"];
    /**
     * Returns a GUID such as xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx.
     *
     * @return New GUID.
     */
    function newGuid(separator) {
        if (separator === void 0) { separator = "-"; }
        // c.f. rfc4122 (UUID version 4 = xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx)
        var oct = "";
        var tmp;
        for (var a = 0; a < 4; a++) {
            tmp = (4294967296 * Math.random()) | 0;
            oct += hexValues[tmp & 0xF]
                + hexValues[tmp >> 4 & 0xF]
                + hexValues[tmp >> 8 & 0xF]
                + hexValues[tmp >> 12 & 0xF]
                + hexValues[tmp >> 16 & 0xF]
                + hexValues[tmp >> 20 & 0xF]
                + hexValues[tmp >> 24 & 0xF]
                + hexValues[tmp >> 28 & 0xF];
        }
        // "Set the two most significant bits (bits 6 and 7) of the clock_seq_hi_and_reserved to zero and one, respectively"
        var clockSequenceHi = hexValues[8 + (Math.random() * 4) | 0];
        return oct.substr(0, 8) + separator + oct.substr(9, 4) + separator + "4" + oct.substr(13, 3) + separator + clockSequenceHi + oct.substr(16, 3) + separator + oct.substr(19, 12);
    }
    Utilities.newGuid = newGuid;
    /**
     * Helper to clone an object.
     */
    function clone(obj) {
        if (!obj) {
            return obj;
        }
        var clone = {};
        forEachKey(obj, function (key, val) {
            clone[key] = "" + val;
        });
        return clone;
    }
    Utilities.clone = clone;
    /**
     * Helper to iterate over all keys in an ovject.
     */
    function forEachKey(obj, iterator) {
        Object.keys(obj).forEach(function (k) {
            iterator(k, obj[k]);
        });
    }
    Utilities.forEachKey = forEachKey;
    /**
     * Helper to find an item in an array.
     */
    function findInArr(arr, callbackFn) {
        return arr.find(callbackFn);
    }
    Utilities.findInArr = findInArr;
})(Utilities = exports.Utilities || (exports.Utilities = {}));
//# sourceMappingURL=utilities.js.map