/// <reference types="node" />
import * as express from "express";
import * as https from "https";
import * as Q from "q";
export declare module HttpsProxy {
    /**
     * The local server options.
     */
    interface ServerOptions {
        /**
         * The local server port.
         */
        serverPort: number;
        /**
         * NOTE: if not defined default will be used.
         */
        serverCallback?: () => {};
    }
    /**
     * The proxy options.
     */
    interface ProxyOptions {
        /**
         * NOTE: if not defined default will be used.
         */
        errorCallback?: (err: any, req: any, res: any) => {};
        /**
         * NOTE: if not defined default will be used.
         */
        responseCallback?: (proxyRes: any, req: any, res: any) => {};
        /**
         * NOTE: if not defined default will be used.
         */
        openCallback?: (proxySocket: any) => {};
        /**
         * NOTE: if not defined default will be used.
         */
        closeCallback?: (res: any, socket: any, head: any) => {};
    }
    /**
     * The local express server interface.
     */
    interface ExpressServer extends express.Express {
        /**
         * The instance of http proxy.
         */
        proxy?: any;
    }
    /**
     * The instance https proxy server
     */
    interface ProxyServer {
        /**
         * The local server.
         */
        server: https.Server;
        /**
         * The proxy server instance.
         */
        app: ExpressServer;
        /**
         * The instance of http proxy.
         */
        proxy?: any;
        /**
         * Sets the default routes, should be called after all other routes are setup.
         */
        initializeDefaultRoutes: () => void;
        /**
         * The local address for the portal patch file.
         */
        patchAddress: string;
    }
    /**
     * Creates a local server running express with a proxy to allow forwarding call to a target host.
     *
     * @param sslOptions The ssl options for the local https server.
     * @param targetHost The target host for the proxy.
     * @param serverOptions The local https server options.
     * @param proxOptions The proxy options.
     * @return A promise that when resolved will return the proxy server.
     */
    function createServerWithProxy(sslOptions: any, targetHost: string, serverOptions: ServerOptions, proxOptions?: ProxyOptions): Q.Promise<ProxyServer>;
    /**
     * Creates a local server running express.
     *
     * @param sslOptions The ssl options for the local https server.
     * @param targetHost The target host for the proxy.
     * @param options The local https server options.
     * @param defaultHandler The default handler for incoming requests. The default handler return 404.
     * @return A promise that when resolved will return the proxy server.
     */
    function createServer(sslOptions: any, targetHost: string, options: ServerOptions, defaultHandler?: express.RequestHandler): Q.Promise<ProxyServer>;
}
