"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpsProxy = void 0;
var bodyparser = require("body-parser");
var express = require("express");
var fs = require("fs");
var https = require("https");
var httpProxy = require("http-proxy");
var Q = require("q");
var cors = require('cors');
var HttpsProxy;
(function (HttpsProxy) {
    "use strict";
    var localHostDns = "localhost";
    var patchJs = fs.readFileSync(__dirname + "/scripts/patch.js").toString();
    ;
    /**
     * Creates a local server running express with a proxy to allow forwarding call to a target host.
     *
     * @param sslOptions The ssl options for the local https server.
     * @param targetHost The target host for the proxy.
     * @param serverOptions The local https server options.
     * @param proxOptions The proxy options.
     * @return A promise that when resolved will return the proxy server.
     */
    function createServerWithProxy(sslOptions, targetHost, serverOptions, proxOptions) {
        return createProxy(sslOptions, targetHost, proxOptions).then(function (proxy) {
            return createServer(sslOptions, targetHost, serverOptions, null).then(function (proxyServer) {
                proxyServer.app.proxy = proxyServer.proxy = proxy;
                return proxyServer;
            });
        });
    }
    HttpsProxy.createServerWithProxy = createServerWithProxy;
    /**
     * Creates a local server running express.
     *
     * @param sslOptions The ssl options for the local https server.
     * @param targetHost The target host for the proxy.
     * @param options The local https server options.
     * @param defaultHandler The default handler for incoming requests. The default handler return 404.
     * @return A promise that when resolved will return the proxy server.
     */
    function createServer(sslOptions, targetHost, options, defaultHandler) {
        var app = express();
        var httpsServer = https.createServer(sslOptions, app);
        app.use(cors());
        app.use(bodyparser.json());
        app.get("/patch.js", function (req, res) {
            var key = req.query.key;
            var out = patchJs.replace("HOST_NAME", targetHost).replace("LOCAL_SERVER_PORT", "" + options.serverPort);
            res.setHeader('content-type', 'text/javascript');
            res.write(out);
            res.end();
        });
        var initializeDefaultRoutes = function () {
            app.use(defaultHandler || function (req, res) {
                var proxy = (req.app || {}).proxy;
                if (proxy) {
                    proxy.proxyRequest(req, res);
                }
                else {
                    res.sendStatus(404);
                }
            });
        };
        httpsServer.on("close", function (res, socket, head) {
            console.log("Client disconnected");
        });
        httpsServer.listen(options.serverPort, localHostDns, options.serverCallback || function () {
            console.log("Express server listening on port https://" + localHostDns + ":" + options.serverPort);
        });
        return Q({
            server: httpsServer,
            app: app,
            initializeDefaultRoutes: initializeDefaultRoutes,
            patchAddress: "https://localhost:" + options.serverPort + "/patch.js"
        });
    }
    HttpsProxy.createServer = createServer;
    function createProxy(sslOptions, targetHost, options) {
        options = options || {};
        var proxy = httpProxy.createProxyServer({
            ssl: sslOptions,
            target: "https://" + targetHost,
            agent: https.globalAgent,
            changeOrigin: true,
            headers: {
                host: targetHost
            }
        });
        proxy.on("proxyRes", options.responseCallback || function (proxyRes, req, res) {
            console.log("RAW Request from the target: ", req.method + " " + req.url);
            console.log("RAW Request headers: ", JSON.stringify(req.headers));
            console.log("RAW Response from the target: ", JSON.stringify(proxyRes.headers));
            console.log("RAW Response status code: ", proxyRes.statusCode);
        });
        proxy.on("error", options.errorCallback || function (err, req, res) {
            res.writeHead(500, {
                "Content-Type": "text/plain"
            });
            res.end("Something went wrong. And we are reporting a custom error message." + err);
        });
        proxy.on("open", (options.openCallback || function (proxySocket) {
            proxySocket.on("data", "foo");
        }));
        proxy.on("close", options.closeCallback || function (res, socket, head) {
            console.log("Client disconnected");
        });
        return Q(proxy);
    }
})(HttpsProxy = exports.HttpsProxy || (exports.HttpsProxy = {}));
;
//# sourceMappingURL=httpsProxy.js.map