/// <reference types="node" />
import { Application, Request, Response } from "express";
import { IncomingMessage, ServerOptions as HTTPServerOptions } from "http";
import { Server as HTTPSServer } from "https";
import { SecureContextOptions, TlsOptions } from "tls";
import { MockFx } from "..";
import { SSLCertificate } from "../system";
declare type ServerOptions = SecureContextOptions & TlsOptions & HTTPServerOptions;
interface LoopbackResponse {
    statusCode: number;
    headers?: Record<string, string>;
    body?: Buffer;
}
export interface MockProxy {
    send: (req: Request, res: Response) => void;
    close: (callback?: () => void) => void;
}
export declare const enum MockHeaders {
    SOURCE = "x-mockfx-source",
    ORIGINAL_HOST = "x-mockfx-original-host",
    RUN_ID = "x-mockfx-run-id",
    TARGET = "x-mockfx-target",
    TEST_ID = "x-mockfx-test-id"
}
export declare function getServerOptions(certificate: SSLCertificate): ServerOptions;
export declare function createHTTPSServer(app: Application, serverOptions: ServerOptions, port: number): Promise<HTTPSServer>;
export declare function stopServer(server: HTTPSServer): Promise<void>;
export declare function createProxy(host: string, serverOptions: ServerOptions, responseCb: (request: IncomingMessage, mockRequest: MockFx.Request) => Promise<void>, getMetadata?: (req: Request, res: IncomingMessage) => Promise<Record<string, any>>, parseResponse?: (res: IncomingMessage, resBody: Buffer) => Promise<any>): MockProxy;
export declare function request(url: string, method?: string, headers?: Record<string, string>, data?: Buffer): Promise<LoopbackResponse>;
/**
 * Parses a message's rawHeaders array into an object in order to preserve casing.
 *
 * @export
 * @param {string[]} rawHeaders
 * @returns {Record<string, string>}
 */
export declare function parseRawHeaders(rawHeaders: string[]): Record<string, string>;
export {};
