"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeFirewallRule = exports.addFirewallRule = exports.loadWinCAs = exports.getSSLCertificate = void 0;
var child_process_1 = require("child_process");
var path_1 = require("path");
var util_1 = require("util");
var logger_1 = require("../logger");
var winCA = require("win-ca");
var caAPI = require("win-ca/api");
var execAsync = (0, util_1.promisify)(child_process_1.exec);
var identity = "MockFx Server";
/**
 * Executes "createCert.ps1" which either returns the existing MockFx certificate and passphrase or creates one if it doesn't exist.
 */
function getSSLCertificate() {
    return __awaiter(this, void 0, void 0, function () {
        var output, certInfo, pfx, passphrase;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    logger_1.logger.debug("Checking existance of SSL certificate.");
                    return [4 /*yield*/, execAsync("powershell -noprofile -File \"".concat((0, path_1.join)(__dirname, "createCert.ps1"), "\" -FriendlyName \"").concat(identity, "\""), { encoding: "utf8" })];
                case 1:
                    output = _a.sent();
                    if (output.stderr) {
                        throw new Error("Could not create SSL certificate.\n\n".concat(output.stderr));
                    }
                    certInfo = output.stdout.trim().split("^");
                    pfx = Buffer.from(certInfo[0], "base64");
                    passphrase = certInfo[1];
                    loadWinCAs();
                    return [2 /*return*/, {
                            pfx: pfx,
                            passphrase: passphrase,
                        }];
            }
        });
    });
}
exports.getSSLCertificate = getSSLCertificate;
/**
 * Injects the server CA into the TLS context since NodeJS doesn't use the Windows cert store
 */
function loadWinCAs() {
    var CAs = [];
    caAPI({
        store: ["root", "ca"],
        format: 1,
        ondata: function (ca) { return CAs.push(ca); },
    });
    winCA.inject("+", CAs);
}
exports.loadWinCAs = loadWinCAs;
/**
 * Adds a firewall rule to open up a local port for mock server.
 *
 * @param port The local port that should be opened
 */
function addFirewallRule(port) {
    return __awaiter(this, void 0, void 0, function () {
        var err_1, cmd;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (port > 65535) {
                        throw new Error("Port out of range.");
                    }
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, removeFirewallRule()];
                case 2:
                    _a.sent();
                    return [3 /*break*/, 4];
                case 3:
                    err_1 = _a.sent();
                    return [3 /*break*/, 4];
                case 4:
                    logger_1.logger.debug('Creating firewall rule');
                    cmd = "netsh advfirewall firewall add rule name=\"".concat(identity, "\" dir=in program=\"").concat(process.execPath, "\" protocol=tcp localport=").concat(port, " action=allow");
                    return [2 /*return*/, new Promise(function (resolve, reject) {
                            (0, child_process_1.exec)(cmd, function (error, stdout, stderr) {
                                if (error) {
                                    reject("Failed to overwrite existing firewall rule.\nError:".concat(error, ",\nSTDOUT:").concat(stdout, ",\nSTDERR:").concat(stderr));
                                }
                                else {
                                    resolve();
                                }
                            });
                        })];
            }
        });
    });
}
exports.addFirewallRule = addFirewallRule;
/**
 * Removes a firewall rule.
 *
 * @param name The name of the rule.
 */
function removeFirewallRule() {
    logger_1.logger.debug('Removing firewall rule');
    var cmd = "netsh advfirewall firewall delete rule name=\"".concat(identity, "\"");
    return new Promise(function (resolve, reject) {
        (0, child_process_1.exec)(cmd, function (error, stdout, stderr) {
            if (error) {
                reject("Failed to remove firewall rule.\nError:".concat(error, ",\nSTDOUT:").concat(stdout, ",\nSTDERR:").concat(stderr));
            }
            else {
                resolve();
            }
        });
    });
}
exports.removeFirewallRule = removeFirewallRule;
//# sourceMappingURL=index.js.map