"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateDeploymentNames = exports.isDeploymentRequest = exports.deploymentRegex = void 0;
var crypto_1 = require("crypto");
exports.deploymentRegex = /\/subscriptions\/.*?\/resourceGroups\/.*?\/providers\/Microsoft\.Resources\/deployments\/(.+?)[?/].*/i;
function isDeploymentRequest(req) {
    return exports.deploymentRegex.test(req.originalUrl);
}
exports.isDeploymentRequest = isDeploymentRequest;
function updateDeploymentNames(requests, req, seenDeployments) {
    var _a, _b;
    var newDeploymentName = req.originalUrl.match(exports.deploymentRegex)[1];
    // We've already updated the requests for this deployment
    if (seenDeployments.indexOf(newDeploymentName.toLocaleUpperCase()) !== -1) {
        return [requests, seenDeployments];
    }
    // Find match by name
    var deployment = requests.find(function (mockRequest) {
        var matches = mockRequest.url.match(exports.deploymentRegex);
        return matches !== null
            && seenDeployments.indexOf(matches[1].toLocaleUpperCase()) === -1
            && matches[1].toLocaleUpperCase() === newDeploymentName.toLocaleUpperCase();
    });
    // Find match by hash
    if (!deployment && ((_b = (_a = req.body) === null || _a === void 0 ? void 0 : _a.properties) === null || _b === void 0 ? void 0 : _b.template)) {
        var templateHash_1 = getObjectHash(req.body.properties.template);
        deployment = requests.find(function (mockRequest) {
            var _a, _b, _c, _d;
            var matches = mockRequest.url.match(exports.deploymentRegex);
            return matches !== null
                && seenDeployments.indexOf(matches[1].toLocaleUpperCase()) === -1
                && ((_b = (_a = mockRequest === null || mockRequest === void 0 ? void 0 : mockRequest.request) === null || _a === void 0 ? void 0 : _a.properties) === null || _b === void 0 ? void 0 : _b.template)
                && getObjectHash((_d = (_c = mockRequest === null || mockRequest === void 0 ? void 0 : mockRequest.request) === null || _c === void 0 ? void 0 : _c.properties) === null || _d === void 0 ? void 0 : _d.template) === templateHash_1;
        });
    }
    // Find first unmatched deployment
    if (!deployment) {
        deployment = requests.find(function (mockRequest) {
            var matches = mockRequest.url.match(exports.deploymentRegex);
            return matches !== null && seenDeployments.indexOf(matches[1].toLocaleUpperCase()) === -1;
        });
    }
    if (!deployment) {
        throw new Error("Unable to match a recorded deployment for this request.");
    }
    var oldDeploymentName = deployment.url.match(exports.deploymentRegex)[1];
    var json = JSON.stringify(requests);
    var newRequests = JSON.parse(replaceAll(json, oldDeploymentName, newDeploymentName));
    seenDeployments.push(oldDeploymentName.toLocaleUpperCase());
    seenDeployments.push(newDeploymentName.toLocaleUpperCase());
    return [newRequests, seenDeployments];
}
exports.updateDeploymentNames = updateDeploymentNames;
function replaceAll(input, searchValue, replaceValue) {
    return input.replace(new RegExp(regexEscape(searchValue), "g"), replaceValue);
}
function regexEscape(str) {
    return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
}
function getObjectHash(json) {
    var keys = getKeys(json);
    return (0, crypto_1.createHash)('sha1').update(keys.join('')).digest('base64');
}
function getKeys(node) {
    if (Array.isArray(node)) {
        return node.reduce(function (acc, el) { return acc.concat(getKeys(el)); }, []);
    }
    else if (Object.prototype.toString.call(node) === '[object Object]') {
        return Object.entries(node).reduce(function (acc, _a) {
            var key = _a[0], val = _a[1];
            return acc.concat(__spreadArray([key], getKeys(val), true));
        }, []);
    }
    return [];
}
//# sourceMappingURL=helpers.js.map