"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createARMTarget = exports.getNormalizedUrl = void 0;
var express_1 = require("express");
var url_1 = require("url");
var batch_1 = require("./batch");
var helpers_1 = require("./helpers");
var invoke_1 = require("./invoke");
var DeploymentReducer_1 = require("./reducers/DeploymentReducer");
var LocationReducer_1 = require("./reducers/LocationReducer");
var helpers_2 = require("../helpers");
var insightsRegex = /\/providers\/microsoft.insights\//i;
var timestampRegex = /\d\d\d\d-\d\d-\d\dT\d\d(%3A|:)\d\d(%3A|:)\d\d(\.\d\d\d)?Z/ig;
var defaultLoadFn = function () { return __awaiter(void 0, void 0, void 0, function () {
    return __generator(this, function (_a) {
        throw new Error("Not implemented");
    });
}); };
function getNormalizedUrl(url) {
    var base = "https://localhost";
    var originalUrl = url;
    var originalUrlParsed = new url_1.URL(originalUrl, base);
    // Removing query parameters we don't care about
    originalUrlParsed.searchParams.delete('');
    var normalizedUrl = originalUrlParsed.toString().replace(base, '');
    // Removing timestamp from Insights requests
    if (insightsRegex.test(normalizedUrl) && timestampRegex.test(normalizedUrl)) {
        normalizedUrl = normalizedUrl.replace(timestampRegex, '');
    }
    return normalizedUrl;
}
exports.getNormalizedUrl = getNormalizedUrl;
function defaultCheckRequest(request, normalizedUrl, mockRequest, context) {
    if (request.method === mockRequest.method &&
        request.headers.commandname === mockRequest.metadata.command) {
        if (normalizedUrl === mockRequest.metadata.normalizedUrl) {
            return true;
        }
        else {
            (0, helpers_2.logErrorIfRequestsDifferOnlyInApiVersion)(normalizedUrl, mockRequest.metadata.normalizedUrl, "Proxied request and mock request have same method (".concat(request.method, ") and commandName (").concat(request.headers.commandname, ")"), context);
        }
    }
    return false;
}
/**
 * If a custom matching function is provided and returns no match, run the api-mismatch heuristic.
 *
 * @param customCheckRequest
 * @returns
 */
function addApiMismatchHeuristic(customCheckRequest) {
    return function (request, normalizedUrl, mockRequest, context) {
        if (customCheckRequest(request, normalizedUrl, mockRequest, context)) {
            return true;
        }
        else {
            (0, helpers_2.logErrorIfRequestsDifferOnlyInApiVersion)(normalizedUrl, mockRequest.metadata.normalizedUrl, '', context);
        }
        return false;
    };
}
function createARMTarget(_a) {
    var _this = this;
    var host = _a.host, _b = _a.loadRequests, loadRequests = _b === void 0 ? defaultLoadFn : _b, _c = _a.storeRequests, storeRequests = _c === void 0 ? defaultLoadFn : _c, onBeforeRequestMatch = _a.onBeforeRequestMatch, customMatchRequest = _a.customMatchRequest;
    var operationReducers = [new DeploymentReducer_1.DeploymentReducer(), new LocationReducer_1.LocationReducer()];
    return {
        name: "ARM",
        hosts: [host],
        loadRequests: loadRequests,
        storeRequests: function (context, requests) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                // Collapse long-running operations
                operationReducers.forEach(function (reducer) {
                    requests = reducer.reduce(requests);
                });
                // Call user instance
                return [2 /*return*/, storeRequests(context, requests)];
            });
        }); },
        getRouter: function () {
            return (0, express_1.Router)()
                .use(function (req, res, next) {
                // ARM requests should always be JSON
                if (!req.headers['content-type'] && req.method !== 'GET' && req.method !== 'HEAD') {
                    req.headers['content-type'] = 'application/json';
                }
                next();
            })
                .use((0, express_1.json)({ type: '*/*' }))
                .use("/batch", batch_1.batchRouter)
                .use("/api/invoke", invoke_1.invokeRouter);
        },
        onBeforeRequestMatch: function (request, context, storedRequests) { return __awaiter(_this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                // Update requests with new deployment names
                if ((0, helpers_1.isDeploymentRequest)(request)) {
                    context.metadata.__seenDeployments = Array.isArray(context.metadata.__seenDeployments)
                        ? context.metadata.__seenDeployments
                        : [];
                    _a = (0, helpers_1.updateDeploymentNames)(storedRequests, request, context.metadata.__seenDeployments), storedRequests = _a[0], context.metadata.__seenDeployments = _a[1];
                }
                if (onBeforeRequestMatch) {
                    return [2 /*return*/, onBeforeRequestMatch(request, context, storedRequests)];
                }
                return [2 /*return*/, storedRequests];
            });
        }); },
        matchRequest: function (req, unmatchedRequests, storedRequests, context) { return __awaiter(_this, void 0, void 0, function () {
            var normalizedUrl, checkRequest, matchedRequest, lastIndex, mockRequest, i, request;
            return __generator(this, function (_a) {
                normalizedUrl = getNormalizedUrl(req.originalUrl);
                checkRequest = customMatchRequest ? addApiMismatchHeuristic(customMatchRequest) : defaultCheckRequest;
                matchedRequest = unmatchedRequests.find(function (mockRequest) { return checkRequest(req, normalizedUrl, mockRequest, context); });
                // If no unmatched request, look through the previously matched requests
                // and find the last one that matches and return that again if no subsequent
                // matched requests altered resources
                if (!matchedRequest) {
                    lastIndex = storedRequests.length - 1;
                    // Find the last previously matched request
                    for (; lastIndex >= 0; lastIndex--) {
                        mockRequest = storedRequests[lastIndex];
                        // Skip requests until we get to the last matched request
                        if (mockRequest.matched && checkRequest(req, normalizedUrl, mockRequest, context)) {
                            matchedRequest = mockRequest;
                            break;
                        }
                    }
                    if (matchedRequest) {
                        // Check if subsequent matched requests were to alter resources
                        for (i = lastIndex + 1; i < storedRequests.length; i++) {
                            request = storedRequests[i];
                            if (request.matched && ['POST', 'PUT', 'DELETE', 'PATCH'].includes(request.method)) {
                                // POSTs to ARG are fine
                                if (request.method === 'POST' && (/\/providers\/Microsoft.ResourceGraph\/resources/i.test(request.url))) {
                                    continue;
                                }
                                // Found a request to alter a resource, so clear the reference to the last matched request
                                matchedRequest = undefined;
                                break;
                            }
                        }
                    }
                }
                return [2 /*return*/, matchedRequest];
            });
        }); },
        getRequestMetadata: function (req) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, {
                        normalizedUrl: getNormalizedUrl(req.originalUrl),
                        command: req.headers.commandname,
                    }];
            });
        }); },
        parseResponse: function (response, responseBody) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (responseBody.length) {
                    return [2 /*return*/, JSON.parse(responseBody.toString("utf8"))];
                }
                return [2 /*return*/, {}];
            });
        }); },
    };
}
exports.createARMTarget = createARMTarget;
//# sourceMappingURL=index.js.map