"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocationReducer = void 0;
/*
    To understand the call patterns please read below.
    https://docs.microsoft.com/azure/azure-resource-manager/management/async-operations
*/
var LocationReducer = /** @class */ (function () {
    function LocationReducer() {
    }
    LocationReducer.prototype.reduce = function (requests) {
        var operations = {};
        var acc = [];
        for (var _i = 0, requests_1 = requests; _i < requests_1.length; _i++) {
            var req = requests_1[_i];
            var currentOperation = getOperationFromPartialURL(operations, req.url);
            if (currentOperation) {
                // This request is part of an operation chain, but we're not finished yet
                if (req.responseHeaders.Location && req.responseCode === 202) {
                    var originalReq = operations[currentOperation];
                    // Update the operation key with new location pointer
                    delete operations[currentOperation];
                    operations[req.responseHeaders.Location] = originalReq;
                    continue;
                }
                // The chain is finished, update the original request to point to the current one
                operations[currentOperation].responseHeaders.Location = currentOperation;
                delete operations[currentOperation];
            }
            else if (req.responseHeaders.Location && !operations[req.responseHeaders.Location]) {
                // New operation starting
                operations[req.responseHeaders.Location] = req;
            }
            acc.push(req);
        }
        return acc;
    };
    return LocationReducer;
}());
exports.LocationReducer = LocationReducer;
function getOperationFromPartialURL(operations, url) {
    return Object.keys(operations).find(function (key) { return key.includes(url); });
}
//# sourceMappingURL=LocationReducer.js.map