"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Certificates = void 0;
var bodyparser = require("body-parser");
var rdfeHelper_1 = require("../rdfeHelper");
var constants_1 = require("../../Helpers/constants");
var urlParams_1 = require("./urlParams");
var Certificates;
(function (Certificates) {
    "use strict";
    /**
     * Initializes the routes.
     *
     * @param app The express app.
     */
    function init(app) {
        /**
         * Add Management Certificate
         */
        app.post("/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/certificates"), bodyparser.text(), function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var certificate = rdfeHelper_1.RdfeHelper.convertXmlToSubscriptionCertificate(req.body);
            if (mockSubscriptionId && certificate) {
                var armManager = req.app.rdfeManager;
                var response = armManager.addManagementCertificate(mockSubscriptionId, certificate);
                response
                    ? res.status(constants_1.Constants.HttpStatusCode.Ok).end()
                    : res.status(constants_1.Constants.HttpStatusCode.BadRequest).end();
            }
            else {
                next();
            }
        });
        /**
         * Delete Management Certificate
         */
        app.delete("/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/certificates/:").concat(urlParams_1.UrlParams.Params.thumbprint), function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockThumbprintId = req[urlParams_1.UrlParams.Params.thumbprint];
            if (mockSubscriptionId && mockThumbprintId) {
                var armManager = req.app.rdfeManager;
                var response = armManager.deleteManagementCertificate(mockSubscriptionId, mockThumbprintId);
                response
                    ? res.status(constants_1.Constants.HttpStatusCode.Ok).end()
                    : res.status(constants_1.Constants.HttpStatusCode.BadRequest).end();
            }
            else {
                next();
            }
        });
        /**
         * Get Management Certificate
         */
        app.get("/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/certificates/:").concat(urlParams_1.UrlParams.Params.thumbprint), function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockThumbprintId = req[urlParams_1.UrlParams.Params.thumbprint];
            if (mockSubscriptionId && mockThumbprintId) {
                var armManager = req.app.rdfeManager;
                var certificate = armManager.getManagementCertificate(mockSubscriptionId, mockThumbprintId);
                certificate
                    ? res.status(constants_1.Constants.HttpStatusCode.Ok).send(rdfeHelper_1.RdfeHelper.convertSubscriptionCertificateToXml(certificate))
                    : res.status(constants_1.Constants.HttpStatusCode.BadRequest).end();
            }
            else {
                next();
            }
        });
        /**
         * List Management Certificates
         */
        app.get("/:".concat(urlParams_1.UrlParams.Params.subscriptionId, "/certificates"), function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            if (mockSubscriptionId) {
                var armManager = req.app.rdfeManager;
                var certificates = armManager.listManagementCertificates(mockSubscriptionId);
                certificates
                    ? res.status(constants_1.Constants.HttpStatusCode.Ok).send(rdfeHelper_1.RdfeHelper.convertSubscriptionCertificatesToXml(certificates))
                    : res.status(constants_1.Constants.HttpStatusCode.BadRequest).end();
            }
            else {
                next();
            }
        });
    }
    Certificates.init = init;
})(Certificates = exports.Certificates || (exports.Certificates = {}));
//# sourceMappingURL=Certificates.js.map