"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RdfeHelper = void 0;
var cheerio = require("cheerio");
var RdfeHelper;
(function (RdfeHelper) {
    "use strict";
    /**
     * Converts a SubscriptionCertificate array into a xml encoded string.
     * @param certificates
     */
    function convertSubscriptionCertificatesToXml(certificates) {
        var str = "<SubscriptionCertificates>\n";
        certificates.forEach(function (cert) { return str += convertSubscriptionCertificateToXml(cert); });
        str += "</SubscriptionCertificates>";
        return str;
    }
    RdfeHelper.convertSubscriptionCertificatesToXml = convertSubscriptionCertificatesToXml;
    /**
     * Converts a SubvscriptionCertificate into a xml encoded string.
     * @param certificate
     */
    function convertSubscriptionCertificateToXml(certificate) {
        var template = "<SubscriptionCertificate>\n<SubscriptionCertificatePublicKey>{0}</SubscriptionCertificatePublicKey>\n<SubscriptionCertificateThumbprint>{1}</SubscriptionCertificateThumbprint>\n<SubscriptionCertificateData>{2}</SubscriptionCertificateData>\n<Created>{3}</Created>\n</SubscriptionCertificate>\n";
        return StringFormat(template, certificate.subscriptionCertificatePublicKey, certificate.subscriptionCertificateThumbprint, certificate.subscriptionCertificateData, certificate.created.toUTCString());
    }
    RdfeHelper.convertSubscriptionCertificateToXml = convertSubscriptionCertificateToXml;
    /**
     * Converts a xml string encoding of SubvscriptionCertificates into a SubscriptionCertificate array.
     * @param xml
     */
    function convertXmlToSubscriptionCertificates(xml) {
        var re = new RegExp("(<SubscriptionCertificate>[\\s\\S]*?<\/SubscriptionCertificate>)", "g");
        var matches = getMatches(xml, re, 1);
        return matches.map(function (certXml) { return convertXmlToSubscriptionCertificate(certXml); });
    }
    RdfeHelper.convertXmlToSubscriptionCertificates = convertXmlToSubscriptionCertificates;
    /**
     * Converts a xml string encoding of a SubscriptionCertificate into the Javascript Object.
     * @param xml
     */
    function convertXmlToSubscriptionCertificate(xml) {
        var $ = cheerio.load(xml, { xmlMode: true });
        return {
            subscriptionCertificatePublicKey: convertXmlToPublicKey($, xml),
            subscriptionCertificateThumbprint: convertXmlToThumbprint($, xml),
            subscriptionCertificateData: convertXmlToData($, xml),
            created: new Date(Date.now())
        };
    }
    RdfeHelper.convertXmlToSubscriptionCertificate = convertXmlToSubscriptionCertificate;
    function convertXmlToPublicKey($, xml) {
        return $('SubscriptionCertificate').find('SubscriptionCertificatePublicKey').text();
    }
    function convertXmlToThumbprint($, xml) {
        return $('SubscriptionCertificate').find('SubscriptionCertificateThumbprint').text();
    }
    function convertXmlToData($, xml) {
        return $('SubscriptionCertificate').find('SubscriptionCertificateData').text();
    }
    function StringFormat(str) {
        var replacements = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            replacements[_i - 1] = arguments[_i];
        }
        return str.replace(/{(\d+)}/g, function (match, num) {
            return typeof replacements[num] != 'undefined'
                ? replacements[num]
                : match;
        });
    }
    /**
     * Deep clone a javascript object
     * @param obj
     */
    function cloneObject(obj) {
        // Handle the 3 simple types, and null or undefined
        if (null == obj || "object" != typeof obj)
            return obj;
        // Handle Date
        if (obj instanceof Date) {
            var copy = new Date();
            copy.setTime(obj.getTime());
            return copy;
        }
        // Handle Array
        if (obj instanceof Array) {
            var copy = [];
            for (var i = 0, len = obj.length; i < len; i++) {
                copy[i] = cloneObject(obj[i]);
            }
            return copy;
        }
        // Handle Object
        if (obj instanceof Object) {
            var copy = {};
            for (var attr in obj) {
                if (obj.hasOwnProperty(attr))
                    copy[attr] = cloneObject(obj[attr]);
            }
            return copy;
        }
        throw new Error("Unable to copy obj! Its type isn't supported.");
    }
    RdfeHelper.cloneObject = cloneObject;
    /**
     * Gets all matches for the nth capturing group.
     * @param str
     * @param regex
     * @param index defaults to 1st capture group
     */
    function getMatches(str, regex, index) {
        if (index === void 0) { index = 1; }
        var matches = [];
        var match;
        while (match = regex.exec(str)) {
            matches.push(match[index]);
        }
        return matches;
    }
})(RdfeHelper = exports.RdfeHelper || (exports.RdfeHelper = {}));
//# sourceMappingURL=rdfeHelper.js.map